package org.apache.geronimo.ejemplos.dbtester.beans;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.sql.DataSource;

import org.apache.geronimo.connector.outbound.ConnectionFactorySource;
import org.apache.geronimo.gbean.AbstractName;
import org.apache.geronimo.gbean.AbstractNameQuery;
import org.apache.geronimo.kernel.GBeanNotFoundException;
import org.apache.geronimo.kernel.InternalKernelException;
import org.apache.geronimo.kernel.Kernel;
import org.apache.geronimo.kernel.KernelRegistry;
import org.apache.geronimo.kernel.NoSuchOperationException;

/**
 * Corazn de la aplicacin. Controla la mayor parte de la lgica de la
 * aplicacin (incluyendo el acceso al Kernel de Geronimo).
 * 
 * @author Lasantha Ranaweera
 * @author Adaptacin a EclipseWTP: Waldo Ramrez Montao
 */
public class DBManagerBean {
    /**
     * Lista de Pools activas.
     */
    private Map poolMap;

    /**
     * Lista de tablas propias de una pool activa.
     */
    private Map tableMap;

    /**
     * Constructor. (1) Construye al HashMap que pretende ser poblado por la
     * lista de pools activas, la cual (2) se pobla con la invocacin del mtodo
     * <b>init()</b> de esta misma clase.
     * 
     */
    public DBManagerBean() {
        poolMap = new HashMap();
        init();
    }

    /**
     * Parte ms importante en esta aplicacin: proporciona acceso al kernel de
     * Geronimo, y as obtiene la lista de las pools de conexiones a bases de
     * datos que se encuentren activas.
     * 
     */
    private void init() {
        Kernel kernel = KernelRegistry.getSingleKernel();
        Set cfList = kernel.listGBeans(new AbstractNameQuery(
                ConnectionFactorySource.class.getName()));

        for (Iterator iterator = cfList.iterator(); iterator.hasNext();) {
            AbstractName name = (AbstractName) iterator.next();
            try {
                Object rs = kernel.invoke(name, "$getResource",
                        new Object[] {}, new String[] {});
                if (rs instanceof javax.sql.DataSource) {
                    DataSource ds = (DataSource) rs;
                    poolMap.put(name.getArtifact().getArtifactId(), ds);
                }
            } catch (GBeanNotFoundException e) {
                e.printStackTrace();
            } catch (NoSuchOperationException e) {
                e.printStackTrace();
            } catch (InternalKernelException e) {
                e.printStackTrace();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Getter para obtener el Map de las pools de conexiones a BD activadas.
     * 
     * @return Map, que contiene las Pools activadas.
     */
    public Map getDeployedPools() {
        return poolMap;
    }

    /**
     * Getter para obtener el Map de la lista de tablas en una BD, a travs de
     * su Pool.
     * 
     * @param poolName
     * @return Map, que contiene las tablas de la BD con pool poolName
     * @throws SQLException
     */
    public Map getTableList(String poolName) throws SQLException {
        tableMap = new HashMap();
        if (poolMap.containsKey(poolName)) {
            DataSource ds = (DataSource) poolMap.get(poolName);
            Connection con = null;
            try {
                con = ds.getConnection();
                DatabaseMetaData metaData = con.getMetaData();
                String[] tableTypes = { "TABLE" };
                ResultSet rs = metaData.getTables(null, null, null, tableTypes);
                while (rs.next()) {
                    String schemaName = rs.getString("TABLE_SCHEM");
                    String tableName = rs.getString("TABLE_NAME");
                    ArrayList tableList = null;
                    if (tableMap.containsKey(schemaName)) {
                        tableList = (ArrayList) tableMap.get(schemaName);
                        tableList.add(tableName);
                    } else {
                        tableList = new ArrayList();
                        tableList.add(tableName);
                    }
                    tableMap.put(schemaName, tableList);
                }
            } catch (SQLException e) {
                throw e;
            } finally {
                if (con != null) {
                    try {
                        con.close();
                    } catch (SQLException e) {
                        e.printStackTrace();
                    }
                }
            }
        }

        return tableMap;
    }

    /**
     * Getter para obtener el contenido de una tabla en la BD
     * 
     * @param poolName
     * @param schemaName
     * @param tableName
     * @return List, de registros en la tabla
     * @throws SQLException
     */
    public List getTableContents(String poolName, String schemaName,
            String tableName) throws SQLException {
        ArrayList matrix = new ArrayList();
        DataSource ds = (DataSource) poolMap.get(poolName);
        Connection con = null;
        con = ds.getConnection();
        String SQL = "SELECT * FROM ";

        if (schemaName == null || schemaName.equals("null")) {
            SQL += tableName; // Problem comes with MySQL
        } else {
            SQL += (schemaName + "." + tableName);
        }
        PreparedStatement pStmt = con.prepareStatement(SQL);
        ResultSet rs = pStmt.executeQuery();
        ResultSetMetaData metaData = rs.getMetaData();
        int columns = metaData.getColumnCount();
        ArrayList header = new ArrayList();
        for (int i = 1; i <= columns; i++) {
            String colName = metaData.getColumnLabel(i);
            header.add(colName);
        }
        matrix.add(header);
        while (rs.next()) {
            ArrayList row = new ArrayList();
            for (int i = 1; i <= columns; i++) {
                row.add(rs.getString(i));
            }
            matrix.add(row);
        }
        return matrix;
    }
}
